/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "MuIPPressure.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace granularRheologyModels
{
    defineTypeNameAndDebug(MuIPPressure, 0);
    addToRunTimeSelectionTable(PPressureModel, MuIPPressure, dictionary);
}
}

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::granularRheologyModels::MuIPPressure::MuIPPressure(const dictionary& dict)
:
    PPressureModel(dict)
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::granularRheologyModels::MuIPPressure::~MuIPPressure()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField> Foam::granularRheologyModels::MuIPPressure::pa
(
         const volScalarField& pf,
	 const dimensionedScalar& Bphi,
	 const dimensionedScalar& rhoa,
	 const dimensionedScalar& da,
	 const dimensionedScalar& rhob,
	 const dimensionedScalar& nub,
	 const volScalarField& magD,
	 const volScalarField& alpha,
	 const dimensionedScalar& alphaMax,
	 const dimensionedScalar& Alphasmall
) const
{
  //
  // MuI inertial regime: pa = (b alpha / (alphaMax - alpha))^2 rhoa da^2 magD^2
  return pow(Bphi*alpha/max(alphaMax - alpha, scalar(1e-3)),2)*rhoa*pow(da,2)*pow(magD,2);

}

// ************************************************************************* //
